function [Logic,SetOfSols,ChangedSols,Last_label]=ExpandCluster(SetOfSols,NewSol,NewClID,Eps,MinPts)
%
% [Logic,SetOfSols,ChangedSols]=ExpandCluster(SetOfSols,NewSol,NewClID,...
%                               Eps,MinPts)
%
%
% NewSol and SetOfSols are structure variables with three fields of Value,
% ClID,and Type where Value, ClID, and Type respectively contain data point 
% values, their cluster ID, and their cumulative adjacency density. 
% This clustering method is a new version of Density Based Spatial 
% Clustering Application with Noise (DBSCAN) which is specifically 
% applicable for problems that dynamically create one datapoint at a time 
% (NewSol). After specifying its cluster ID, NewSol is added to SetOfSols. 
% Determination of cluster label is facilitated by two parameters of Eps 
% (maximum distance from new solution) and MinPts (minimum number of 
% points around new solution including itself in a distance less than or 
% equal to Eps). For example, 1000 data points have been created so far and
% now a new point is introduced to the search space after perturbation. 
% Assume that the cluster label of the new point is required and 
% there exist clusters and noises (zero cluster ID) in previous points. 
% This function checks all possible conditions for the new point and 
% finally updates all datapoints' labels if any change is made due to the 
% presence of the new point. The updated points in terms of cluster labels
% assigned to ChangedSols. ChangedSols is again a structure variable with
% the same fields as NewSol or SetOfSols.
%
% Inputs:
%       SetOfSols: Data/solutions information structure (created so far)
%
%               SetOfSols.Value: Data/Solution matrix (rows solutions,
%               columns decision variables)
%
%               SetOfSols.ClID: Solutions' cluster labels
%
%               SetOfSols.Type: Cumulative adjacency density from the
%               beginning of optimization for the archived solutions
%
%       NewSol: The new generated solution with the same structure and
%               fields as SetOfSols
%       
%       NewClID: New Cluster Label which equals the last label plus one.
%                NewClID is included in the input since new cluster is
%                formed due to introducing NewSol to the search space.
%
%       Eps: neighbourhood domain for a solution to update the adjacency
%                density around a solution.
%
%       MinPts: minimum number of reachable solutions to consider the
%                solution as core
%
% Outputs:
%       Logic: 1 (if a new cluster is created by NewSol)
%              0 (if no new cluster is created)
%                 (even if new point joins previously created clusters or
%                 if it leads to merging two previously separated clusters,
%                 Logic is again equal to 0)
%
%       SetOfSols: Updated version of all solutions (including NewSol) and
%                  their new labels if changed by NewSol.
%
%       ChangedSols: All the solutions with their new cluster IDs or their  
%                  updated cluster IDs are stored here.


% Searching for the reachable solutions from the new generated solution
% (NewSol) based on the pre-determined density parameters of MinPts and
% Eps. The RegionQuery function is duing this task. All the nearby
% solutions are saved in Seeds.
Logic=0;
% If factor equals one it means do not update SetOfSols at the end
factor=0; 
[Seeds,SetOfSols, NewSol]=RegionQuery(SetOfSols,NewSol,Eps);
% Add new solution to the last row of the set of archived solutions.
SetOfSols.Value=[SetOfSols.Value; NewSol.Value];
SetOfSols.ClID=[SetOfSols.ClID; NewSol.ClID];
SetOfSols.Type=[SetOfSols.Type; NewSol.Type];
% searching for any point around NewSol to be considered as core point
CoresLabels1=find(Seeds.Type>=MinPts);
if ~isempty(CoresLabels1)
    Logic1=1;
else
    % if there is no core point around, the new solution is considered as
    % noise.
    Logic1=0;
    factor=1;
end
if isequal(Logic1,1)
    % flag1 and flag2 are used for if-else statements in the following.
    flag1=0;
    flag2=0;
    % Move all the potential core points from Seeds to CoreSeeds.
    CoreSeeds.Value=Seeds.Value(CoresLabels1,:);
    CoreSeeds.ClID=Seeds.ClID(CoresLabels1);
    CoreSeeds.Type=Seeds.Type(CoresLabels1);
    % Find potential core with zero cluster label among the cores that are
    % nearby NewSol.
    ZeroID=find(CoreSeeds.ClID==0);
    % Save the labels of the cores nearby NewSol with distinct nonzero
    % cluster IDs in H.
    H= unique(CoreSeeds.ClID);
    H(H==0)=[];
    Pts=[];
    % First off the possibility of having an unclassified core should be
    % checked. If the minimum cumulative adjacency density is met, a new
    % cluster is formed. This is accomplished by doing a RegionQuery search
    % around all the potential core points with zero cluster ID. If the
    % number of unclassified points around the potential core points is
    % equal to or higher than MinPts, a new cluster is formed.
    for i=1:length(ZeroID)
        % check the neighborhood of the candidate zero ID cores one by one.
        Pot_Core.Value=CoreSeeds.Value(ZeroID(i),:);
        Pot_Core.ClID=CoreSeeds.ClID(ZeroID(i));
        Pot_Core.Type=CoreSeeds.Type(ZeroID(i));
        [Core_Neighbor{i},~,~]=RegionQuery(SetOfSols,Pot_Core,Eps);
        % If the candidate unclassified core is reachable from a previously
        % clustered core do not proceed to forming new cluster (break the 
        % for loop) since this case is either a cluster expansion or a
        % cluster merge situation.
        loc2= find(Core_Neighbor{i}.Type>=MinPts);
        ClID=unique(Core_Neighbor{i}.ClID(loc2));
        ClID(ClID==0)=[];
        if ~isempty(ClID)
            flag2=1;
            break;
        end
        % Find all the previously clustered solutions nearby the candidate
        % zero ID core.
        row1{i}=find(Core_Neighbor{i}.ClID>0);
        % If the unclassified potential core point includes border
        % points with previously assigned labels, they have to be
        % removed and the number of remaining points that have no
        % cluster ID should be counted.
        Pts(i)=Pot_Core.Type-size(row1{i},1);
    end
    % Find the candidate zero ID core that still meets the minimum
    % cumulative adjacency density after removing the nearby nonzero ID
    % solutions.
    CoreLocs=find(Pts>=MinPts);
    % If the potential core satisfies the minimum cumulative adjacency
    % density and it is not reachable from a previously classified core,
    % form a new cluster.
    if ~isempty(CoreLocs) && isequal(flag2,0) % Situations #5, #6, and #7
        C=[];
        temp1=[];
        for i=1:length(CoreLocs)
            % Remove all the solutions with nonzero cluster ID nearby the
            % zero ID core(s).
            Core_Neighbor{CoreLocs(i)}.Value(row1{CoreLocs(i)},:)=[];
            Core_Neighbor{CoreLocs(i)}.ClID(row1{CoreLocs(i)})=[];
            Core_Neighbor{CoreLocs(i)}.Type(row1{CoreLocs(i)})=[];
            C=[C;Core_Neighbor{CoreLocs(i)}.Value];
            temp1=[temp1;Core_Neighbor{CoreLocs(i)}.Type];
        end
        % Move all the members of the new cluster to Seeds and assign
        % NewClID label to them.
        [C,ia,~]=unique(C,'rows','stable');
        Seeds.Value=C;
        Seeds.ClID=repmat(NewClID,size(Seeds.Value,1),1);
        Seeds.Type=temp1(ia);
        Logic=1;
        flag1=1;
    end
    if isequal(flag1,0) && isempty(H)
        % If new cluster is not formed and there is no clustered core point
        % around new solution, NewSol is a noise.
        factor=1;
    elseif isequal(flag1,0) && ~isempty(H)
        % If new cluster is not formed and there exist clussified cores
        % nearby NewSol.
        CoreSeeds2=CoreSeeds; % move all the nearby potential cores to CoreSeeds2.
        
        % Delete the zero ID points that are considered as core after doing
        % RegionQuery search because they for sure do not satisfy the
        % minimum cumulative adjacency density in this point of the code.
        % Check if the classified points nearby NewSol are still meeting
        % the minimum cumulative adjacency density after deleting the
        % points nearby the core with inconsistent nonzero cluster labels.
        CoreSeeds2.Value(ZeroID,:)=[];
        CoreSeeds2.ClID(ZeroID)=[];
        CoreSeeds2.Type(ZeroID)=[];
        for i=1:size(CoreSeeds2.Value,1)
            Pot_Core.Value=CoreSeeds2.Value(i,:);
            Pot_Core.ClID=CoreSeeds2.ClID(i);
            Pot_Core.Type=CoreSeeds2.Type(i);
            [Core_Neighbor{i},~,~]=RegionQuery(SetOfSols,Pot_Core,Eps);
            Y=find(~ismember(Core_Neighbor{i}.ClID, [0 H'])==1);
            Check=Pot_Core.Type-length(Y);
            if Check < MinPts
                % If the candidate core nearby the NewSol cannot meet the
                % MinPts after discarding the points nearby the candidate
                % core that have inconsistent labels with core cluster ID, 
                % it cannot expand the cluster and remove it from the
                % CoreSeeds.
                Y= find(ismember(CoreSeeds.Value,Pot_Core.Value,'rows')==1);
                CoreSeeds.Value(Y,:)=[];
                CoreSeeds.ClID(Y)=[];
                CoreSeeds.Type(Y)=[];
            end
        end
        % Update H vector.
        H=unique(CoreSeeds.ClID);
        H(H==0)=[];
        if isempty(H)
            % If H is empty, it means there is no core around and NewSol is
            % a noise.
            factor=1;
        elseif length(H)==1
            % IF there are cores in the neighborhood of new solution and
            % the cores have the same ClID>0. There can also be potential
            % cores with zero cluster label along with nonzero ID cores in
            % CoreSeeds.
            
            % Ignoring the clustered points of other clusters with
            % IDs different from those in CoreSeeds, if the cores still
            % satisfy the minimum cumulative adjecency density, a cluster
            % expansion occurs and new solution will join the cluster.
            % Otherwise, the new solution is a noise.
            C=[];
            temp1=[];
            for i=1:size(CoreSeeds.Value,1)
                Pot_Core.Value=CoreSeeds.Value(i,:);
                Pot_Core.ClID=CoreSeeds.ClID(i);
                Pot_Core.Type=CoreSeeds.Type(i);
                [Neighbor,~,~]=RegionQuery(SetOfSols,Pot_Core,Eps);
                Y=find(Neighbor.ClID~=H & Neighbor.ClID~=0); % Situation #13 (C-D)
                Check=Pot_Core.Type-length(Y);
                Neighbor.Value(Y,:)=[]; % situations #9, #12, and #14. For explanation, start from #14
                Neighbor.ClID(Y)=[]; % situations #9, #12, and #14
                Neighbor.Type(Y)=[]; % situations #9, #12, and #14
                if Check >= MinPts % situations #9, #12, and #14
                    C=[C;Neighbor.Value]; % situations #9, #12, and #14
                    temp1=[temp1;Neighbor.Type]; % situations #9, #12, and #14
                end
            end
            if ~isempty(C)
                [C,ia,~]=unique(C,'rows','stable');
                Seeds.Value=C;
                Seeds.ClID=repmat(H,size(Seeds.Value,1),1);
                Seeds.Type=temp1(ia);
            else
                factor=1;
            end
        else
            % IF there are cores with different ClID>0 in the neighborhood of new
            % solution. Two or more than two clusters are about to merge and a new
            % solution is created in the region between them. The NewSol is
            % reachable from potential core points with different nonzero
            % cluster IDs. If the candidate core points nearby the new
            % solution are themselves reachable to classified points with
            % labels other than what is indicated in H, they should be
            % discarded. If the candidate classified cores still meet the
            % minimum adjacency density after discarding and if the cores
            % in CoreSeeds are density reachable, merge their clusters into
            % one. Otherwise, the new solution joins the nearest cluster.
            Index=[];
            for i=1:size(CoreSeeds.Value,1)
                Pot_Core.Value=CoreSeeds.Value(i,:);
                Pot_Core.ClID=CoreSeeds.ClID(i);
                Pot_Core.Type=CoreSeeds.Type(i);
                if Pot_Core.ClID==0
                    [Core_Neighbor,~,~]=RegionQuery(SetOfSols,Pot_Core,Eps);
                    Y=find(~ismember(Core_Neighbor.ClID, [0 H'])==1);
                    Check=Pot_Core.Type-length(Y);
                    if Check < MinPts
                        Index=[Index i];
                    end
                end
            end
            if ~isempty(Index)
                CoreSeeds.Value(Index,:)=[];
                CoreSeeds.ClID(Index)=[];
                CoreSeeds.Type(Index)=[];
            end
            [CoreSeeds.ClID,arrangement]=sort(CoreSeeds.ClID,'descend');
            CoreSeeds.Value=CoreSeeds.Value(arrangement,:);
            CoreSeeds.Type=CoreSeeds.Type(arrangement);
            CoreDistance=squareform(pdist(CoreSeeds.Value,'euclidean'));
            CoreDistance=triu(CoreDistance,1);
            [Z,AB]=find(CoreDistance<=Eps & CoreDistance>0);
            if ~isempty(Z)
                for k=1:length(Z)
                    if CoreSeeds.ClID(Z(k))~=0 && CoreSeeds.ClID(AB(k))~=0
                        SetOfSols.ClID(SetOfSols.ClID==CoreSeeds.ClID(AB(k)))=CoreSeeds.ClID(Z(k));
                        CoreSeeds.ClID(AB(k))=CoreSeeds.ClID(Z(k));
                    else
                        if CoreSeeds.ClID(Z(k))==0
                            [Neighbor,~,~]=RegionQuery(SetOfSols,struct('Value',CoreSeeds.Value(Z(k),:),'ClID',CoreSeeds.ClID(Z(k)),'Type',CoreSeeds.Type(Z(k))),Eps);
                            Y=find(Neighbor.ClID~=0);
                            Neighbor.Value(Y,:)=[];
                            Neighbor.ClID(Y)=[];
                            Neighbor.Type(Y)=[];
                            Y=find(ismember(SetOfSols.Value,Neighbor.Value,'rows')==1);
                            SetOfSols.ClID(Y)=CoreSeeds.ClID(AB(k));
                            CoreSeeds.ClID(Z(k))=CoreSeeds.ClID(AB(k));
                        end
                        if CoreSeeds.ClID(AB(k))==0
                            [Neighbor,~,~]=RegionQuery(SetOfSols,struct('Value',CoreSeeds.Value(AB(k),:),'ClID',CoreSeeds.ClID(AB(k)),'Type',CoreSeeds.Type(AB(k))),Eps);
                            Y=find(Neighbor.ClID~=0);
                            Neighbor.Value(Y,:)=[];
                            Neighbor.ClID(Y)=[];
                            Neighbor.Type(Y)=[];
                            Y=find(ismember(SetOfSols.Value,Neighbor.Value,'rows')==1);
                            SetOfSols.ClID(Y)=CoreSeeds.ClID(Z(k));
                            CoreSeeds.ClID(AB(k))=CoreSeeds.ClID(Z(k));
                        end
                    end
                end
                H=unique(CoreSeeds.ClID);
                for j=1:length(H)
                    Y= find(CoreSeeds.ClID==H(j));
                    for k=1:length(Y)
                        [Neighbor,~,~]=RegionQuery(SetOfSols,struct('Value',CoreSeeds.Value(Y(k),:),'ClID',CoreSeeds.ClID(Y(k)),'Type',CoreSeeds.Type(Y(k))),Eps);
                        YY= find(Neighbor.ClID==0);
                        if ~isempty(YY)
                            YYY= find(ismember(SetOfSols.Value,Neighbor.Value(YY,:),'rows')==1);
                            SetOfSols.ClID(YYY)=H(j);
                        end
                    end
                end
                factor=1;
            else
                Y= CoreSeeds.ClID==0;
                CoreSeeds.Value(Y,:)=[];
                CoreSeeds.ClID(Y)=[];
                CoreSeeds.Type(Y)=[];
                for k=1:size(CoreSeeds.Value,1)
                    %Distance(k)=pdist([CoreSeeds.Value(k,:);NewSol.Value],'minkowski',1.3);
                    Distance(k)=pdist([CoreSeeds.Value(k,:);NewSol.Value],'euclidean');
                end
                Y= find(Distance==min(Distance));
                Pot_Core.Value=CoreSeeds.Value(Y(1),:);
                Pot_Core.ClID=CoreSeeds.ClID(Y(1));
                Pot_Core.Type=CoreSeeds.Type(Y(1));
                [Seeds,~,~]=RegionQuery(SetOfSols,Pot_Core,Eps);
                Y=find(Seeds.ClID~=Pot_Core.ClID & Seeds.ClID~=0);
                Seeds.Value(Y,:)=[];
                Seeds.ClID(Y)=[];
                Seeds.Type(Y)=[];
                Seeds.ClID=Pot_Core.ClID*ones(1,size(Seeds.Value,1));
            end
        end
    end
end
if factor==0
    U=find(ismember(SetOfSols.Value,Seeds.Value,'rows')==1);
    SetOfSols.ClID(U)=repmat(unique(Seeds.ClID),length(U),1);
    [SetOfSols.ClID, Last_label]=relabel1(SetOfSols.ClID);
    ChangedSols=struct('Value',SetOfSols.Value(U,:),'ClID',SetOfSols.ClID(U),'Type',SetOfSols.Type(U));
    NewSol.ClID=SetOfSols.ClID(end);
end
if factor==1
    [SetOfSols.ClID, Last_label]=relabel1(SetOfSols.ClID);
    NewSol.ClID=SetOfSols.ClID(end);
    ChangedSols=struct('Value',NewSol.Value,'ClID',NewSol.ClID,'Type',NewSol.Type);
end
end